# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from mudlib import mudlibapi

import time

class Hatrack(mudlibapi.MudObject):
  """
  The hatrack is not very exciting.  It allows people to hang up their hat
  (which we randomly figure out) and also to take their hat back off the
  hatrack.  They can see what other hats are hung up as well.
  """
  def __init__(self):
    self._names = ["hatrack", "rack"]
    self._short = "A hatrack."
    self._desc = ("The hatrack is filled with an assortment of hats from " +
                  "people who are probably milling around in the tavern.  " +
                  "If you hang up your hat, don't forget to fetch it " +
                  "before you leave!")

    self._hats = ["hat", "cap", "wool hat", "sombrero", "helmet", "fez"]
    self._hanged_hats = []

  def getDesc(self, mob, name):
    """
    Returns the description as well as the number of hats currently
    on the hatrack.
    """
    if name in self._names:
      if len(self._hanged_hats) == 0:
        hats = "There are no hats hanging on the hatrack."
      elif len(self._hanged_hats) == 1:
        hats = "There is one hat hanging on the hatrack."
      else:
        hats = ("There are %d hats hanging on the hatrack." %
                len(self._hanged_hats))
      return self._desc + "\n\n" + hats

    if name == "hats":
      if len(self._hanged_hats) == 0:
        return "There are no hats hanging on the hatrack."
      else:
        return ("Currently hanging on the hatrack: " + 
                ", ".join(self._hanged_hats))

    return None

  def cmd_hang(self, mob, world, input):
    """ Allows mobs to hang their hat."""
    hat = getattr(mob, "_hanged_hat", 0)
    if hat == 0:
      mob._hanged_hat = mudlibapi.random_choice(self._hats)
      mob.writeln("You hang your " + mob._hanged_hat + " on the hatrack.  " +
                  "Don't forget to fetch it before you go!")
      self._hanged_hats.append(mob._hanged_hat)
      return 1
    else:
      mob.writeln("You see your %s on the hatrack where you hung it " +
                  "before." % mob._hanged_hat)
      return 1

  def cmd_fetch(self, mob, world, input):
    """ Allows mobs to retrieve their hat."""
    hat = getattr(mob, "_hanged_hat", 0)
    words = input.split(" ", 1)
    if (not hat == 0 and len(words) > 1 and (
          words[1] == "hat" or words[1] in self._hats)):
      mob.writeln("You fetch your %s from the hatrack." % mob._hanged_hat)
      self._hanged_hats.remove(mob._hanged_hat)
      mob._hanged_hat = 0
      return 1
    return 0

class Table(mudlibapi.MudObject):
  """
  Right now this does nothing.  But soon you'll be able to sit down (if
  there's room), stand up, talk with other people at the table, and such.
  Technically it's a room because it has exits and such.  And when you
  "sit down" you get moved to the room.
  """
  def __init__(self):
    self._names = ["table"]
    self._short = "A table."
    self._desc = ("A long table you could sit down at.  Clearly it's " +
                  "been here for some time as there are names carved in " +
                  "its surface that look older than time itself.")

  def cmd_stand(self, mob, world, input):
    pass

  def cmd_sit(self, mob, world, input):
    pass


class GuestBookDesk(mudlibapi.MudObject):
  def __init__(self):
    self._names = ["desk"]
    self._short = "A desk."
    self._desc = ("It's a small desk used to hold a guestbook.")

  def getDesc(self, mob, name):
    if name in ["guestbook", "book"]:
      book = ("This guestbook has many pages in it--and it's likely " +
              "this tavern has had many before it.")

      # this is a little sneaky...
      world = getattr(mob, "_world", None)
      if world:
        output = book + "\n"
        if len(world._log) > 0:
          output = "\n".join(world._log) + "\n"
        for mem in world._mobiles:
          output += "  %s - still on\n" % mem.getShort(None, mem._names[0])
        return output
      return book

    if name in self._names:
      return self._desc
    return None

class GrandfatherClock(mudlibapi.MudObject):
  def __init__(self):
    self._names = ["clock"]
    self._short = "A grandfather clock."
    self._desc = ("This clock belonged to Neil's great grandfather who " +
                  "opened this tavern back in ... .  It's been ticking " + 
                  "for many many years under a power that few of the " +
                  "finest craftsmen in the land know how to harness " +
                  "anymore.  The clock stands ominously against the wall, " +
                  "ticking the seconds of countless millenia by....")

    mudlibapi.register_hb(self)

  def getDesc(self, mob, name):
    if name in self._names:
      thetime = time.strftime("The time is:  %I:%M:%S %p (%Z)", time.localtime())
      return self._desc + "\n\n" + thetime
    return None

  def heartbeat(self, my_world):
    """
    The clock heartbeat ticks the millenia by.  It spams the world
    with the striking of a new hour.
    """
    timetuple = time.localtime()

    oldtime = getattr(self, "_time", -1)
    if oldtime == -1:

      self._time = timetuple[3]
      return

    if self._time < timetuple[3]:
      self._time = timetuple[3]
      mess = mudlibapi.Message(("The clock strikes %d.\n" % (self._time % 12)) +
                             ("DONG! " * self._time) + "\n",
                             datatype=EVENT)
      my_world.spamworld(mess)
