# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
This is an api for driver stuff.  This just makes things easier to code
in the driver.  Driver things should use this API to access other driver
things--it centralizes some of the business logic into a standard 
interface.  In addition, the mudlib should use the driverapi to access
things in the driver.
"""
import sys, traceback, threading

# we do this so we can epydoc this module
try:
  import engine
except:
  pass

DISCONNECT_QUIT = "quit"
DISCONNECT_LINKDEATH = "linkdeath"

HOOK_FIRST = 1
HOOK_LAST = 99

def start_thread(threadname, threadfunc):
  """
  Starts a thread with the name and func given.

  @param threadname: the name of the thread to start
  @type  threadname: string

  @param threadfunc: the function for the thread to work on
  @type  threadfunc: function

  @returns: the thread just created
  @rtype: Thread
  """
  # create and initialize the new thread and stick it in our list
  log("new thread: %r" % threadfunc)
  t = threading.Thread(group=None, target=threadfunc, name=threadname)
  t.setDaemon(1)
  t.start()
  return t


def log(message):
  """
  Logs stuff to stdout.

  @param message: the message to log to stdout.
  @type  message: string
  """
  if get_engine()._verbose == 1:
    print message

def get_engine():
  """
  Retrieves the Engine singleton.

  @returns: the Engine object singleton
  @rtype: Engine
  """
  return engine.get_engine()

def add_manager(name, manager):
  """
  Adds a manager to the Engine.

  @param name: the name of the manager to add
  @type  name: string

  @param manager: the manager to add
  @type  manager: Manager
  """
  get_engine().addManager(name, manager)

def get_manager(name):
  """
  Retrieves a manager from the engine.

  @param name: the name of the manager to retrieve
  @type  name: string

  @returns: the manager or None
  @rtype: Manager
  """
  return get_engine().getManager(name)

def register_hook(hookname, func, place=None):
  """
  Registers a function with a given hook.  If the hook doesn't exist
  then it instantiates the hook.

  @param hookname: the name of the hook
  @type  hookname: string

  @param func: the function to call when that hook is spammed
  @type  func: function

  @param place: the function will get this place in the call
      order.  functions with the same place specified will get
      arbitrary ordering.  defaults to LAST.
  @type  place: int
  """
  if place:
    get_engine().registerHook(hookname, func, place)
  else:
    get_engine().registerHook(hookname, func)


def print_traceback(message=""):
  """
  Convenience method for grabbing the traceback, formatting it, 
  and piping it to the logs.

  @param message: any message you want to pass to the user--this
      gets printed first
  @type  message: string
  """
  exc = "".join(traceback.format_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2]))

  if message:
    message = message + "\n" + exc
  else:
    message = exc
  print message

import filesystem
instantiate = filesystem.instantiate
