# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Holds the Engine class which manages the following things:

  - registered managers (who get nice things like startup and shutdown 
    calls)
  - hooks and all registered functions
  - the thread that manages the event queue and execution of
    events therein

X{Hooks<hooks>} are pretty interesting.  Our hooks are created when
something first registers with the hook.  If nothing registers with
the hook, then calling spamHook will do nothing.  Hooks can be
regular hooks (list of registered functions) or priority hooks.
Spamming a hook can also take a mapping function enabling you
to mutate the data between function calls.  You can also spam a hook
in such a way that each individual function is called in a try/except
block for added safety.  See spamHook for more information.

The driver creates the following important hooks:

B{startup} - the X{startup hook} for when the mud starts up.  Things should
get initialized, but wait to spin off threads until this hook is spammed.
By the time this hook is spammed, you're guaranteed to have the rest
of the mud bootstrapped.  

args tuple is (options dict,)

B{connect} - the X{connect hook} gets spammed when something connects
to the server.  

args tuple is (Connection,)

B{disconnect} - the X{disconnect hook} gets spammed when something
disconnects from the mud--this could be from the network thread (the
user dropped his/her connection) or from an in-game source (the user
quit or was booted).  

args tuple is (Connection, reason_int)

B{user_input} - the X{user_input hook} gets spammed whenever a
Connection has incoming input.  Depending on the connection state,
this is typically a complete line of input.  

args tuple is (Connection, input string)
"""
import Queue, sys, traceback, time
import network, event, driverapi, filesystem

my_engine = None

def get_engine():
  """
  This will instantiate a new Engine object if none has been created.
  Otherwise it returns the existing singleton Engine object.

  @returns: the Engine object
  @rtype: Engine
  """
  global my_engine
  if my_engine == None:
    my_engine = Engine()
  return my_engine

class Engine:
  """
  This is where the magic happens.  The Engine holds the series
  of managers and the event loop.  It executes events one by
  one while kicking off hooks that other things can be registered
  with.  This allows the various components of the mud driver
  to keep to themselves and yet remain coordinated on a single
  thread of execution.
  """
  def __init__(self):
    self._event_queue = Queue.Queue(0)
    self._worker = None

    # options passed in via the startup script
    self._options = {}

    # managers
    self._managers = {}

    # map of all the hooks we're managing.  string -> utils.PriorityQueue
    self._hook_map = {}

    # whether or not to be verbose (FIXME - switch to log4py)
    self._verbose = 0

    # flag to indicate that we need to shutdown
    self._shutdown = 0

  def __repr__(self):
    return "<ENGINE>"

  def setOptions(self, options):
    """
    Sets the local _options dict.  These are options that were configured 
    at startup usually by the command line.

    @param options: the startup options dict
    @type  options: dict
    """
    self._options = options
    self._verbose = options.get("verbose", 0)

  def startup(self):
    """
    Things happen in the following order:

      1. instantiates the network MasterServer
      2. calls the X{bootstrap} function in the mudlib package __init__
      3. spams the startup hook
      4. then launches into the main loop
    """
    # instantiate managers we need
    self.addManager("network", network.MasterServer(self))

    # load modules in mudlib packages (they need to be packages)
    for mem in self._options["mudlib"]:
      init_module = filesystem.get_module(mem + ".__init__")
      init_module.bootstrap(self)


    from threading import currentThread
    self._worker = currentThread()

    # fix this so that it's more configurable
    driverapi.log("Host: %s" % self._options["host"])
    driverapi.log("Port: %s" % self._options["port"])
    driverapi.log("Startroom: %s" % self._options["startroom"])

    # we're all started up, so we run "startup" on all the
    # managers so they can spin off threads and do all that
    # crazy stuff
    self.spamHook("startup", {"options": self._options}, tryexcept=0)

    # start up the repo thread
    driverapi.start_thread("repo", self._repothread)

    # this is our main loop for handling events
    while self._shutdown == 0:
      ev = self._event_queue.get(1)
      # driverapi.log("handling: %s" % str(ev))

      try:
        ev.execute(self)

      except KeyboardInterrupt:
        driverapi.log("KeyboardInterrupt called.")
        break

      except SystemExit:
        break

      except Exception, e:
        driverapi.log("exception: %s" % e)
        traceback.print_exc()

        try:
          if hasattr(ev, '_conn'):
            ev._conn.write("A bug goes whizzing by!\nexception: %s\n" % e)
        except Exception, e2:
          print e2

    print "engine exited."

  def _repothread(self):
    """
    This thread sleeps for 5 seconds, wakes up, and dumps a kick event
    on the event queue.  It isn't needed if there are heartbeats happening,
    but since the KickEvent isn't all that exciting, it's not a big
    deal.

    This allows us to block on the event queue and still catch 
    KeyboardInterrupts.
    """
    import time
    while self._shutdown == 0:
      try:
        time.sleep(5)
        event.KickEvent().enqueue()
      except KeyboardInterrupt:
        return
      except SystemExit:
        return
      except:
        driverapi.print_traceback("TimedEventManager: hiccup")

  def shutdown(self):
    """
    Shuts down the mud.  First, it shuts down the event handling
    thread.  Then it spams the shutdown hook giving everything
    registered a chance to shutdown cleanly.
    """
    self._shutdown = 1
    self.spamHook("shutdown", {}, tryexcept=1)
    driverapi.log("Shutting down.")


  # --------------------------------------------------
  # event handling
  # --------------------------------------------------
  def enqueue(self, ev):
    """
    Enqueues an event for later execution.

    @param ev: the event object
    @type  ev: Event
    """
    self._event_queue.put(ev)

  def addManager(self, name, manager):
    """
    Adds a manager to the existing managers.  

    Note: managers are stored by manager name.  So if two managers have 
    the same name, then one of them will stomp on the other.

    @param name: the name of the manager to add
    @type  name: string

    @param manager: the manager object to add
    @type  manager: Manager
    """
    if self._managers.has_key(name):
      print ("warning: we have a manager by name of '%s': '%s'" % 
             (name, repr(self._managers[name])))
    self._managers[name] = manager

  def getManager(self, name):
    """
    Returns the manager or None if there is no manager by that name

    @param name: the name of the manager to retrieve
    @type  name: string

    @returns: the manager or None
    @rtype: Manager
    """
    if self._managers.has_key(name):
      return self._managers[name]
    return None
    

  # ---------------------------------------
  # hook stuff
  # ---------------------------------------
  def getHookList(self):
    """
    Returns a list of hooks that are currently in existence.

    @returns: the list of hooks in existence
    @rtype: list of strings
    """
    return self._hook_map.keys()

  def spamHook(self, hookname, args, mappingfunc=None, tryexcept=0):
    """
    Spams registered functions for a hook with a given set of arguments.
    If the hook doesn't exist, then we don't do anything.

    @param hookname: the name of the hook
    @type  hookname: string

    @param args: the arguments to pass
    @type  args: tuple

    @param mappingfunction: function whose output will be passed to the next
        function in the hook.  Must take two arguments: the previous 
        arglist and the return from the previous function.
    @type  mappingfunction: function

    @param tryexcept: whether (1) or not (0) to execute each
        func in a try/except block.  this has the advantage
        of making sure that if one func dies, it doesn't prevent
        the rest of them from executing.
    @type  tryexcept: boolean
    """
    if self._hook_map.has_key(hookname):
      self._hook_map[hookname].spamhook(input=args, mappingfunc=mappingfunc, tryexcept=tryexcept)

  def registerHook(self, hookname, func, place=driverapi.HOOK_LAST):
    """
    Registers a function with a given hook.  If the hook doesn't
    exist, then it instantiates the hook.

    @param hookname: the name of the hook
    @type  hookname: string

    @param func: the function to call when that hook is spammed
    @type  func: function

    @param place: the function will get this place in the call
        order.  functions with the same place specified will get
        arbitrary ordering.  defaults to driverapi.HOOK_LAST.
    @type  place: int
    """
    if not self._hook_map.has_key(hookname):
      self._hook_map[hookname] = Hook(hookname)

    self._hook_map[hookname].register(func, place)

  def unregisterHook(self, hookname, func):
    """
    If the hook exists, unregisters the func from the hook.

    @param hookname: the name of the hook
    @type  hookname: string

    @param func: the function to remove from the hook
    @type  func: function
    """
    if self._hook_map.has_key(hookname):
      self._hook_map[hookname].unregister(func)

  def getHook(self, hookname, create=1):
    """
    Retrieves a hook by the name of hookname.  If the hook does
    not exist, then it creates the hook.

    @param hookname: the name of the hook
    @type  hookname: string

    @param create: whether (1) or not (0) to create a Hook if
        one doesn't already exist.  if create is 0 and the Hook
        does not exist, then we return None.
    @type  create: boolean    

    @returns: the existing or newly created Hook instance
    @rtype: Hook
    """
    if self._hook_map.has_key(hookname):
      return self._hook_map[hookname]

    if create==1:
      self._hook_map[hookname] = Hook(hookname)
      return self._hook_map[hookname]
    else:
      return None

class Hook:
  """
  Represents a (possibly empty) sequence of user-defined
  functions.  Provides users with the opportunity of reacting
  to events internal to Lyntin.  All functions take a single
  argument which is a dict.

  Read through the hooks.py file for more information on the 
  hooks that come with Lyntin as well as which arguments they 
  take in the arg tuple.
  """
  def __init__(self, name, mapper=lambda x,y:x):
    """
    Initializes.

    @param mapper: function whose output will be passed to the next
        function in the hook.  Must take two arguments: the previous 
        arglist and the return from the previous function.
    @type  mapper: function
    """
    # this is the master priority list
    self._functionlist = {}

    # this gets recomputed everytime someone registers or unregisters 
    # a hook
    self._orderedlist = []

    # the function whose output will be passed to the next function 
    # in the hook
    self._mapper = mapper

    # the name of the hook
    self._name = name

  def __repr__(self):
    """
    Returns the name of this hook.

    @returns: name
    @rtype: string
    """
    return self._name

  def setFilterMapper(self, newmapper):
    """
    Sets the filter mapper to a new mapper.

    @param newmapper: the function whose output will be passed
        to the next function in the hook.  Takes two arguments:
        the previous arglist and the return from the previous 
        function.
    @type  newmapper: function
    """
    self._mapper = newmapper

  def createOrderedList(self):
    """
    Goes through the functionlist and generates the
    orderedlist.  This helps save some cycles every time
    we spam the hook.
    """
    priorities = self._functionlist.keys();
    priorities.sort()

    self._orderedlist = []

    for priority in priorities:
      for mem in self._functionlist[priority]:
        self._orderedlist.append(mem)
    

  def spamhook(self, input={}, 
        mappingfunc=lambda x,y:x, 
        donefunc=lambda x:0,
        defaultfunc=None, 
        tryexcept=0):
    """
    Sends out input to all the registrants of a hook.

    @param input: the dict of arguments that gets passed to
        each function in the hook.  the actual arguments differs
        from hook to hook.
    @type  input: dict of arguments

    @param mappingfunc: function whose output will be passed to the next
        function in the hook.  Must take two arguments: the previous 
        arglist and the return from the previous function.
    @type  mappingfunc: function

    @param donefunc: this function tests whether we're done doing
        what we're doing.  If this function returns true (1) then
        we'll drop out of the loop.
    @type  donefunc: function

    @param defaultfunc: if this is set and we finish going through all
        the functions in the chain and none of them have returned something
        that satisfies the donefunc, then we'll execute the defaultfunc
        with the latest version of the input dict.
    @type  defaultfunc: function

    @param tryexcept: whether (1) or not (0) to execute each
        func in a try/except block.  this has the advantage
        of making sure that if one func dies, it doesn't prevent
        the rest of them from executing.
    @type  tryexcept: boolean

    @return: varies
    @rtype:  varies
    """
    driverapi.log("spam: %r %r" % (self, input))

    mappingfunc = mappingfunc or self._mapper

    listing = self._orderedlist

    if tryexcept == 1:
      for mem in listing:
        try:
          output = mem(input)
          if donefunc(output) == 1:
            break
          input = mappingfunc(input, output)
        except:
          driverapi.print_traceback("hook kicked up exception")

    else:
      for mem in listing:
        output = mem(input)
        if donefunc(output) == 1:
          break
        input = mappingfunc(input, output)

    if defaultfunc != None and callable(defaultfunc) and donefunc(output) != 1:
      return defaultfunc(input)
        
    return output

  def unregister(self, func):
    """
    Tries to remove a registrant from a hook--does pretty well.

    @param func: the function to unregister
    @type  func: function
    """
    for priority in self._functionlist.keys():
      if func in self._functionlist[priority]:
        self._functionlist[priority].remove(func)
        if len(self._functionlist[priority]) == 0:
          del self._functionlist[priority]

    self.createOrderedList()

  def register(self, func, place=driverapi.HOOK_LAST):
    """
    Registers a function with a hook.

    func 
    should be a callable function.  place is optional--it allows 
    you to put yourself earlier in the hook lineup.

    @param func: the function to call when the hook is spammed
    @type  func: function

    @param place: the function will get this place in the call
        order.  functions with the same place specified will get
        arbitrary ordering.  defaults to driverapi.HOOK_LAST.
    @type  place: int
    """
    if not callable(func):
      exported.write_error("Function %s not callable." % repr(func))
      return

    if self._functionlist.has_key(place):
      self._functionlist[place].append(func)
    else:
      self._functionlist[place] = [func]

    self.createOrderedList()

  def clear(self):
    """
    Clears the functionlist.
    """
    self._functionlist = {}
    self._orderedlist = []

  def count(self):
    """
    Returns how many functions are registered with this hook.

    @returns: the number of functions registered
    @rtype: int
    """
    return len(self._functionlist)
