# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Holds the Event superclass and a few subclasses that are used in
the driver.  These classes are what makes everything go since this 
is an event driven mud.

The philosophy of events are that they allow us to run code that
could potentially die without having it kill the whole mud.  It
gives us a cheap way of serializing tasks and (when used correctly)
breaking down those big tasks into smaller chunks so that it appears
that the mud runs with no lag.

It should be noted that events are not protected against infinite 
loops (yet).
"""
import driverapi

class Event:
  """
  The event superclass.  All events should subclass this event and 
  implement the "execute" and "__str__" methods (at a minimum).
  """
  def __init__(self):
    pass

  def execute(self, my_engine):
    """
    This is the method that gets called after the event has been
    plucked off the Queue and is being handled.  It takes a
    Engine object so it knows how to access things.

    Exceptions that are kicked up and not handled by the event
    will get handled by the loop that dequeues and executes
    events.  But in general, it's probably not a good thing to
    be throwing around unhandled exceptions.

    @param my_engine: the Engine instance
    @type  my_engine: Engine
    """
    pass

  def __str__(self):
    """
    String representing the event.  This is useful for verbose
    mode where we log all the events we've handled.
    """
    return ""

  def __repr__(self):
    return self.__str__()

  def enqueue(self):
    """
    Enqueues the current Event in the engine's event queue for later
    execution.
    """
    # driverapi.log("enqueuing: %r" % self)
    driverapi.get_engine().enqueue(self)

class InputEvent(Event):
  """
  The InputEvent handles all incoming user input.  It farms it out
  to the Player object's handleInput method.
  """
  def __init__(self, conn, input):
    Event.__init__(self)
    self._conn = conn
    self._input = input

  def __str__(self):
    return 'InputEvent: "%s" for "%s"' % (self._input, str(self._conn))

  def execute(self, my_engine):
    my_engine.spamHook("userinput", (self._conn, self._input))

class HookEvent(Event):
  """
  When information needs to be passed to registered functions of a hook,
  this allows us to defer execution of that spam event into its own
  event horizon.
  """
  def __init__(self, hookname, args):
    """
    Initializes the HookEvent.

    @param hookname: the name of the hook to spam
    @type  hookname: string

    @param args: the arguments to send to the functions registered
        with the hook in question
    @type  args: tuple
    """
    self._hookname = hookname
    self._args = args

  def __str__(self):
    return "HookEvent: %s" % self._hookname

  def execute(self, my_engine):
    """ Execute."""
    my_engine.spamHook(self._hookname, self._args)


class KickEvent(Event):
  """
  This is a KickEvent for the repo thread which just makes sure
  everything is running squarely and such.  At the moment, the
  KickEvent does nothing.  At some point it may do various cleanup
  activities based on what beat it's on.
  """
  def __init__(self):
    Event.__init__(self)

  def __str__(self):
    return "Kick."

  def execute(self, my_engine):
    pass
