# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
This is the network module which holds the MasterServer and the Connection
objects and handles all socket interface things.

It handles connections and disconnections as well as user input.  It
kicks off appropriate hooks and events accordingly.

A X{connection<handling connections>} happens whenever the master socket 
is tickled.

Connections are handled this way:

  1. the Master socket gets tickled and generates a Connection object
     which wraps around the socket object
  2. the network thread tosses a HookEvent onto the event queue
     which will send a message to the "connect" Hook with the connection 
     as an argument

  3. (some time passes...)

  4. the HookEvent is executed and sends a message to the "connect" Hook
     with the connection as an argument--functions that don't want
     the connection to happen will mark the Connection object as
     such with a _do_not_connect = 1
  5. at the end of the hook (the last item), the network checks for
     the _do_not_connect flag::
         
        if _do_not_connect == 0:
          1. locks the connections list
          2. adds the connection to the list
          3. unlocks the connections list
     
        else:
          1. sends a HookEvent for "disconnect" with the Connection object

A X{disconnect<handling disconnects>} can happen for a variety of reasons:

  1. they drop their connection (linkdeath) which causes an Exception in
     the network thread
  2. they quit out which happens in the event handling thread
  3. a connection occurs, but someone marks it as _do_not_connect so
     it immediately gets disconnected

When a disconnect happens:

  1. a HookEvent is created with the Connection disconnecting and 
     why it's being disconnected (QUIT or LINKDEATH)

  2. (some time passes...)

  3. at the beginning of the hook (the first item), the network removes 
     the connection from the list
  4. the mudlib does whatever it does with disconnects
"""
import socket, select, threading
import event, driverapi, manager

_MASTER_SOCKET = "MASTER"

class MasterServer(manager.Manager):
  """
  The MasterServer handles all the incoming network connections and
  holds a series of Connection objects.
  """
  def __init__(self, newengine):
    self._engine = newengine
    self._options = newengine._options

    self._master = None
    self._conns = []
    self._shutdown = 0

    self._conns_lock = threading.Lock()

    driverapi.register_hook("startup", self.startup_hook)

    driverapi.register_hook("connect", self.connect_hook, 100)
    driverapi.register_hook("disconnect", self.disconnect_hook, 100)

    driverapi.register_hook("shutdown", self.shutdown1_hook, -1)
    driverapi.register_hook("shutdown", self.shutdown2_hook, 100)
    

  def __repr__(self):
    return "<MASTER SERVER>"

  def startup_hook(self, args):
    """
    Initializes the MasterServer.  It'll bind to sockets and
    do all that other stuff at this point.
    """
    host = self._options["host"]
    port = int(self._options["port"])
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    # we try to sit on this and attempt to keep rebinding
    import time
    bound = 0
    attempts = 0
    while bound == 0:
      try:
        s.bind((host, port))
        bound = 1
      except Exception, e:
        attempts += 1
        if attempts == 3:
          raise e
        print "Port is busy--trying again in 10 seconds."
        time.sleep(10)

    del time

    s.listen(1)
    self._master = Connection(self._engine, s, _MASTER_SOCKET)

    self._conns = []
    self._conns.append(self._master)
    
    self._thread = driverapi.start_thread("network", self.mainloop)
    print "Network started."

  def mainloop(self):
    """
    The main loop for picking up data off of the master and other
    socket connections.  We then pass things off to hooks to be handled.
    """
    while self._shutdown == 0:
      fi = []

      conns = list(self._conns)
      fns = map(lambda x:x.getSocket(), conns)
      try:
        fi = select.select(fns, [], [], 1)[0]
      except:
        continue
      allconns = filter(lambda x,y=fi:x.getSocket() in y, conns)

      for conn in allconns:
        if conn._addr == _MASTER_SOCKET:
          newsock, newaddr = conn._sock.accept()
          newconn = Connection(self._engine, newsock, newaddr)
          event.HookEvent("connect", (newconn,)).enqueue()

        else:
          new_data = None
          try:
            new_data = conn._sock.recv(1024) 

          except SystemExit:
            self._shutdown = 1

          except Exception, e:
            driverapi.log("exception: %s with conn %s" % (e, repr(conn)))

          if new_data:
            conn.handleNetworkData(new_data)

          else:
            # this helps to alleviate a bunch of disconnect events for
            # the same connection
            if getattr(conn, "_disconnect", 0) != 1:
              conn._disconnect = 1
              if conn in self._conns:
                self._conns.remove(conn)
              try: conn.close()
              except: pass
              event.HookEvent("disconnect", (conn, driverapi.DISCONNECT_LINKDEATH)).enqueue()
              driverapi.log("Disconnect.")


  def connect_hook(self, args):
    """
    Handles adding the connection to the list of connections we maintain
    and poll from.
    """
    conn = args[0]

    if getattr(conn, "_do_not_connect", 0) == 0:
      driverapi.log("New connection.")
      self._conns.append(conn) 

    else:
      driverapi.log("Rejected connection.")
      event.HookEvent("disconnect", (conn, driverapi.DISCONNECT_LINKDEATH)).enqueue()

  def disconnect_hook(self, args):
    """
    Handles disconnects.
    """
    conn = args[0]
    if conn in self._conns:
      self._conns.remove(conn)
    try: conn.close()
    except: pass

  def shutdown1_hook(self, args):
    """
    Ends the MasterServer network thread.
    """
    self._shutdown = 1

  def shutdown2_hook(self, args):
    """
    Flushes and closes all remaining socket connections.  Then we shut
    down the master socket.  This should be done after the rest of the
    server is shut down so that players get a chance to see what's going
    on (any messages).
    """
    for mem in self._conns:
      if mem._addr == _MASTER_SOCKET:
        continue
      try:
        mem.write("Server shutting down.\n")
        mem.close()
      except Exception, e:
        print "hic: %s" % e

    try: 
      self._master.getSocket().close()
    except Exception, e:
      driverapi.log("closing down master socket: '%s'" % e)
    driverapi.log("network shut down.")


class Connection:
  """
  The connection object handles dealing with the user on a socket
  level.  Here we write data to the user and handle data the user
  has sent us.
  """
  def __init__(self, newengine, newsock, newaddr=''):
    self._engine = newengine
    self._sock = newsock
    self._sock.setblocking(0)      # non-blocking
    self._buffer = []
    self._addr = newaddr
    self._state = 0

    self._name = ""
    self._sex = ""

  def __repr__(self):
    return "Conn: " + repr(self._addr)

  def __str__(self):
    return repr(self._addr)

  def close(self):
    """
    Shuts down the socket for the Connection ob.
    """
    if not self._sock:
      return
    self._sock.shutdown(2)
    self._sock.close()
    self._sock = None
    
  def write(self, data):
    """
    Writes data to the socket.

    @param data: the data to write to the user
    """
    if not self._sock or not data: return

    data = data.replace("\n", "\r\n")
    self._sock.send(data)

  def getSocket(self):
    """
    Returns the socket for this connection.

    @returns: the socket for this connection
    @rtype: Socket
    """
    return self._sock

  def handleNetworkData(self, new_data):
    """
    Handles incoming network data.  It'll buffer it and read it until
    we hit a carriage return at which point we'll handle the line as
    a whole.

    @param new_data: the new data they just typed at us
    @type  new_data: string
    """
    for c in new_data:
      if (c == chr(8) or c == chr(127)):
        if self._buffer:
          self._buffer = self._buffer[:-1]
        continue
      elif ord(c) > 127:
        continue
      elif c == "\n":
        event.InputEvent(self, "".join(self._buffer)).enqueue()

        self._buffer = []
        continue
      elif c == "\r" or c == chr(0):
        pass
      else:
        self._buffer.append(c)
