# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
This has a series of utility functions that aren't related to specific
classes in the application.  They fall into the following classes:
string manipulation and argument parsing.
"""
import string, re, time

def chomp(text):
  """
  Removes all '\\r' and '\\n' from the input string.

  @param text: the text to chomp
  @type  text: string

  @returns: the chomped text
  @rtype: string
  """
  text = text.replace("\n", "")
  text = text.replace("\r", "")
  return text


def parse_args(args):
  """
  Takes in a list of args and parses it out into a hashmap of arg-name 
  to value(s).

  @param args: the list of command-line arguments
  @type  args: list of strings

  @return: list of tuples of (arg, value) pairings
  @rtype: list of tuples of (string, string)
  """
  i = 0
  optlist = []
  while (i < len(args)):

    if args[i][0] == "-":
      if (i+1 < len(args)):
        if args[i+1][0] != "-":
          optlist.append((args[i], args[i+1]))
          i = i + 1
        else:
          optlist.append((args[i], ""))
      else:
        optlist.append((args[i], ""))

    else:
      optlist.append(("", args[i]))

    i = i + 1
  return optlist

class PriorityQueue:
  """
  This is a pretty basic priority queue.
  """
  def __init__(self):
    # holds the maps of priorities to items
    self._prioritymap = {}

    # the ordered list of items
    self._orderedlist = []

    # whether or not our orderedlist is dirty
    self._dirty = 0

  def _generateList(self):
    """
    Goes through the prioritymap and generates an orderedlist.  This
    saves cycles since it puts the ordering of the list up front
    rather than when the orderedlist is retrieved.
    """
    priorities = self._prioritymap.keys();
    priorities.sort()

    self._orderedlist = []

    for priority in priorities:
      for mem in self._prioritymap[priority]:
        self._orderedlist.append(mem)

  def add(self, func, priority=99):
    """
    Adds a function to the prioritymap and marks the PriorityQueue as
    "dirty" which means it needs to regenerate the ordered list before
    it hands it out.

    @param func: the function to call when the hook is spammed
    @type  func: function

    @param priority: the function will get this place in the call
        order.  functions with the same priority specified will get
        arbitrary ordering.  defaults to 99.
    @type  priority: int
    """
    if not callable(func):
      exported.write_error("Function %s not callable." % repr(func))
      return

    if self._prioritymap.has_key(priority):
      self._prioritymap[priority].append(func)
    else:
      self._prioritymap[priority] = [func]

    self._dirty = 1

  def remove(self, func):
    """
    Removes a func from the priority map.

    @param func: the function to unregister
    @type  func: function
    """
    for priority in self._prioritymap.keys():
      if func in self._prioritymap[priority]:
        self._prioritymap[priority].remove(func)

        if len(self._prioritymap[priority]) == 0:
          del self._prioritymap[priority]

        break

    self._dirty = 1
  
  def getList(self):
    """
    Retrieves the list.  It might regenerate it if the prioritymap
    has been adjusted since the last time we regenerated the list.
    """
    if self._dirty == 1:
      self._generateList()
    return self._orderedlist

  def count(self):
    """
    Returns how many functions are in the list.

    @returns: the number of functions registered
    @rtype: int
    """
    return len(self.getList())
