# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Defines the CommandManager which manages commands in the world.

Currently command function names should start with I{cmd_} (but
don't have to) and should have a signature like this::

   def cmd_commandname(mob, my_world, input)

where:

  - I{mob} is the Mobile that initiated the command
  - I{my_world} is the World object
  - I{input} is what the mobile executed
"""
import inspect
from driver import manager

class _CommandData:
  """
  Holds data relating to a command.  It's a helper class.
  """
  def __init__(self):
    self._name = ""
    self._func = None
    self._argparser = None
    self._fqn = ""

  def __repr__(self): return self._name
  def __str__(self): return self._name

  def setName(self, name): self._name = name
  def getName(self): return self._name
  def setNameAdjusted(self, name): self._name_adjusted = name
  def getNameAdjusted(self): return self._name_adjusted
  def setFunc(self, func): self._func = func
  def getFunc(self): return self._func
  def setArgParser(self, ap): self._argparser = ap
  def getArgParser(self): return self._argparser
  def setFQN(self, fqn): self._fqn = fqn
  def getFQN(self): return self._fqn

class CommandManager(manager.Manager):
  """ 
  The CommandManager holds a series of _CommandData objects
  and methods to manipulate and use them.  Stringbean developers
  can add their own commands to Stringbean.

  There can be multiple CommandManagers--though the World should
  have them all.
  """
  def __init__(self):
    self._commands = {}

  def getCommands(self):
    """
    Returns a list of the commands we have registered.

    @return: all the commands that have been registered
    @rtype:  list of strings
    """
    return self._commands.keys()

  def addCommand(self, name, func, helptext=""):
    """
    Registers a command.

    @param name: the name of the command to add
    @type  name: string

    @param func: the function that handles the command
    @type  func: function

    @param helptext: the help text for this command for the user
    @type  helptext: string

    @raise ValueError: if the function is uncallable
    @raise Exception: if the argument spec for the command is unparseable
    """
    if not callable(func):
      raise ValueError, "%s is uncallable." % name

    cd = _CommandData()

    # add the command to the command list
    cd.setFunc(func)

    # toss the command thing in the list
    self._commands[name] = cd

    # deal with the help text
    # if not helptext:
    #   if func.__doc__:
    #     helptext = inspect.getdoc(func)
    #   else:
    #     helptext = "\nThis command has no help."

    if name[0] == "^":
      cd.setNameAdjusted(name[1:])
    else:
      cd.setNameAdjusted(name)

    # fqn = driverapi.add_help(cd.getNameAdjusted(), helptext)
    # cd.setFQN(fqn)
        
  def removeCommand(self, name):
    """
    Removes a command (and the help text) for whatever reasons.

    @param name: the name of the command to remove
    @type  name: string

    @return: 0 if no command was found, 1 if the command was removed
        succesfully.
    @rtype: boolean
    """
    if self._commands.has_key(name):
      cd = self._commands[name]
      del self._commands[name]
      # try:
      #   driverapi.remove_help(cd.getFQN())
      # except:
      #   pass
      return 1
    return 0

  def getCommand(self, name):
    """
    Returns the function for a given command name.

    @param name: the name of the command to retrieve
    @type  name: string

    @return: the function in question or None
    @rtype:  function
    """
    if self._commands.has_key(name):
      return self._commands[name].getFunc()

    return None
