# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
This holds the mobile hierarchy starting with Mobile.  Mobile has two
main subclasses: NPC and Player.

It also holds supporting functions for mobiles.
"""
import object, message, utils

MALE = "m"
FEMALE = "f"

class Mobile(object.MudObject):
  """
  The NPC and Player superclass.  This class should be subclassed
  for all "thinking" objects on the mud.  Any object that moves around
  or thinks or says things should implement mobile.

  The mudlibapi has L{random_choice} and L{random_number} functions
  to aid in random choice making.
  """
  def __init__(self):
    object.MudObject.__init__(self)
    self._names = ["mobile"]
    self._short = "A mobile."
    self._desc = "A mobile."
    self._sex = MALE

    self._location = None

  def word_wrap(self, text, indent=0, firstline=0):
    """
    Word-wrapping is really only useful to the player.  But it helps 
    for all mobiles to have it.  In this case, we just return the text 
    we got with no transformation on it.

    @param text: the text to wrap
    @type  text: string

    @param indent: the number of spaces to indent lines
    @type  indent: int

    @param firstline: whether (1) or not (0) to indent the first line
    @type  firstline: boolean

    @returns: the wrapped string
    @rtype: string
    """
    return text

  def handleInput(self, world, input):
    """
    Allows mobiles to "do" things the way players do them by using
    the regular command_handler.

    @param world: the world object
    @type  world: World

    @param input: the input to handle ("say Hello there stranger!")
    @type  input: string
    """
    ret = command_handler(self, world, input)

class NPC(Mobile):
  """
  Basic NPC class.
  """
  def __init__(self):
    Mobile.__init__(self)
    self._names = ["joe"]
    self._short = "joe npc"
    self._desc = "A regular looking NPC."

  def heartbeat(self, world):
    """
    Most NPCs that want to do things on their own without
    reacting to players poking and prodding them will likely
    want to implement this method and then register with the
    World.
    """
    pass

  def setLocation(self, room):
    """
    This assumes we just moved somewhere.  So we set our location
    and then force a look.

    @param room: the room we just moved to
    @type  room: Room
    """
    self._location = room
    room._contents.append(self)


class Player(Mobile):
  """
  Basic player class.  This class connects to the Connection class.
  This class handles in-game mud stuff while the Connection class
  handles socket/network stuff.
  """
  def __init__(self, conn, my_world):
    Mobile.__init__(self)

    self._names = [conn._name]
    self._short = conn._name
    self._desc = "Wholly non-descript."
    self._sex = conn._sex
    self._world = my_world

    self._conn = conn
    conn._player = self

    self._history = {}

  def setLocation(self, room):
    """
    This assumes we just moved somewhere.  So we set our location
    and then force a look.

    @param room: the room we just moved to
    @type  room: Room
    """
    self._location = room
    room._contents.append(self)
    command_handler(self, self._world, "look")

  def logHistory(self, data, datatype):
    """
    Takes a message and logs it to our history of messages in the
    right place.

    @param data: a string to log in our history
    @type  data: string

    @param datatype: the type of the data
    @type  datatype: int
    """
    # add in the history
    hist = self._history.get(datatype, [])
    hist.append(data)
    self._history[datatype] = hist

    # truncate to the last 20
    if len(self._history[datatype]) > 20:
      self._history[datatype] = self._history[datatype][-20:]

  def getHistory(self, datatype):
    """
    Retrieves history of a specific datatype.

    @param datatype: the type of the data
    @type  datatype: int

    @returns: the history of that datatype
    @rtype: list of strings
    """
    return self._history.get(datatype, [])

  def write(self, data):
    """
    Writes data to the connection object we have.

    @param data: the data to write to the user.
    @type  data: string
    """
    if type(data) == type(""):
      self._conn.write(data)
    else:
      mess = utils.wrap_text(data._message, 70, data._indent, data._firstline)
      self._conn.write(mess)
      self.logHistory(mess, data._datatype)

  def writeln(self, data):
    """
    Appends an LF and then calls write.

    @param data: the data to write to the user.
    @type  data: string or Message
    """
    if type(data) == type(""):
      self.write(data + "\n")
    else:
      data._message = data._message + "\n"
      self.write(data)

  def handleInput(self, world, input):
    """
    Handles user input by farming it out to the command_handler func to 
    handle.  If the command isn't recognized (if the handler returns a 0) 
    we send the user a generic error message.  We send the user a prompt 
    here as well.

    @param world: the world object
    @type  world: World

    @param input: the input the user typed
    @type  input: string
    """
    # FIXME - this is simplistic because I didn't want to spend the
    # time to design a solid command parser at this stage.  it's 
    # somethiing that requires a lot of thought and planning.
    ret = command_handler(self, world, input)

    if len(input) > 0 and not ret:
      self.write("What?\n")

    self.write("> ")


# list of standard aliases
DEFAULT_ALIASES = {"n": "north",
                   "s": "south",
                   "e": "east",
                   "w": "west",
                   "ne": "northeast",
                   "nw": "northwest",
                   "se": "southeast",
                   "sw": "southwest",
                   "u": "up",
                   "d": "down",
                   "l": "look"}

DIRECTIONALS = ["north", "south", "east", "west", "northeast", "northwest", "southeast", "southwest", "up", "down"]

def command_handler(mob, world, input):
  """
  Takes a mobile derivative, a world object, and an input
  string and sees if we want to handle it here.  If we
  handle it, then we return a 1.  Otherwise we return a 0.

  @param mob: a Mobile derivative
  @type  mob: Mobile

  @param world: the World object
  @type  world: World

  @param input: the input line
  @type  input: string

  @returns: 1 if we handled it, 0 if we did not
  @rtype: boolean
  """
  global DEFAULT_ALIASES, DIRECTIONALS

  comm = input.split(" ", 1)[0]

  # expand the alias if there is one
  if DEFAULT_ALIASES.has_key(comm):
    comm = DEFAULT_ALIASES[comm]

  # handle exits first
  newroom = mob._location.getExit(comm)
  if newroom:
    oldroom = mob._location
    newroom = mob._location.getExit(comm)
    if newroom:
      newroom = world.loadRoom(newroom)
      if newroom:
        _move(mob, oldroom, newroom, comm)
        return 1

  # handle exit issues next
  if comm in DIRECTIONALS:
    mob.write("You can't go that way.\n")
    return 1

  # commands on objects in mob's inventory
  for mem in mob._contents:
    func = getattr(mem, "cmd_%s" % comm, None)
    if func != None:
      ret = func(mob, world, input)
      if ret == 1:
        return 1

  # commands on objects in mob's location
  if mob._location:
    for mem in mob._location._contents:
      func = getattr(mem, "cmd_%s" % comm, None)
      if func != None:
        ret = func(mob, world, input)
        if ret == 1:
          return 1

    # commands on mob's location
    func = getattr(mob._location, "cmd_%s" % comm, None)
    if func != None:
      return func(mob, world, input)

  # handle commands
  func = world._commandmanager.getCommand(comm)
  if func != None:
    return func(mob, world, input)

  # handle emotes
  func = world._emotemanager.getCommand(comm)
  if func != None:
    return func(mob, world, input)

  return 0


def _move(pl, oldroom, newroom, direction):
  """
  Handles moving a player from one room to another.
  """
  mess = message.Message("You go %s.\n\n" % direction, message.EVENT, pl)
  pl.write(mess)

  if oldroom:
    if pl in oldroom._contents:
      oldroom._contents.remove(pl)
    mess._message = "%s goes %s.\n" % (pl.getName(), direction)
    oldroom.spamroom(mess, [pl])

  mess._message = "%s arrives.\n" % pl.getName()
  newroom.spamroom(mess, [pl])

  pl.setLocation(newroom)
