# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
This holds the base object MudObject.  From this object everything
is derived which allows everything in the mud to have a basic series
of properties and interfaces making manipulation of mud objects
a bit easier.
"""
class MudObject:
  """
  The base object for all the mud stuff.
  """
  def __init__(self):
    self._names = ["object"]
    self._short = "An indescribable object."
    self._desc = "An indescribable object."
    self._location = None
    self._contents = []

  def init(self, args):
    """
    This allows you to pass instantiation arguments to objects
    when they're being constructed.  So you could have a box class
    and pass in the color of the box in the args tuple when
    you instantiate it.

    @param args: tuple of arguments
    @type  args: tuple
    """
    pass

  def getShort(self, mob, name):
    """
    Returns the one-line short description for the object.

    @param mob: the mobile asking for a description
    @type  mob: mud object

    @param name: the name of the thing to be described
    @type  name: string

    @returns: a description of the thing or None if that isn't this
        thing's name
    @rtype: string
    """
    if name in self._names:
      return self._short
    return None

  def getDesc(self, mob, name):
    """
    Returns the description for that name for a given mob.

    @param mob: the mobile asking for a description
    @type  mob: mud object

    @param name: the name of the thing to be described
    @type  name: string

    @returns: a description of the thing or None if that isn't this
        thing's name
    @rtype: string
    """
    if name in self._names:
      return self._desc
    return None
    
  def setLocation(self, room):
    """
    Sets the location of this object so it knows where it is.

    @param room: the Room where this object is located (though
        it could easily be on a mobile or in a backpack on
        a mobile)
    @type  room: MudObject
    """
    self._location = room
    if self not in room._contents:
      room._contents.append(self)

  def getName(self):
    """
    Returns the first name in the names list for this object.

    @returns: the name of the object
    @rtype: string
    """
    return self._names[0]

  def getNames(self):
    """
    Returns all the names for this object.

    @returns: the names for this object
    @rtype: list of strings
    """
    return self._names

  def heartbeat(self, world):
    """
    Objects can have heartbeats too--however, unless the
    heartbeat is registered with the world, it won't do anything.
    """
    pass

  def __str__(self):
    return repr(self.getName())

  def save(self):
    pass

  def write(self, data):
    """
    Most objects won't do anything with this method, but it
    does allow objects to listen in on things.

    @param data: the data to write to the object.
    @type  data: string or Message
    """
    pass

  def writeln(self, data):
    """
    Most objects won't do anything with this method, but it
    does allow objects to listen in on things.  This one
    also appends an LF.

    @param data: the data to write to the object.
    @type  data: string or Message
    """
    pass
