# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
This is named parser, but it doesn't yet hold a "parser".  Instead it
just provides us a central way of sourcing in all the global commands
and emotes and feeding them to the respective CommandManagers.

Commands and emotes are defined in mini-modules in the commands/
and emotes/ directories.
"""
import world, message, utils
import mudlibapi
import os, glob

def _write(pl, text):
  """
  Helper function for generating a regular Message object with no
  indent.

  @param pl: the player to write to
  @type  pl: Mobile

  @param text: the text to send
  @type  text: string
  """
  pl.write(message.Message(text + "\n", indent=0, firstline=0))

def emote_no_target(source, source_text, room_text):
  """
  Handles emotes which are executed with no target.  

  @param source: the source
  @type  source: Mobile

  @param source_text: the text the source will see.  ex: "You smile."
  @type  source_text: string

  @param room_text: the text the rest of the room sees.  ex: "%s smiles at %t."
  @type  room_text: string
  """
  mess = message.Message(source_text + "\n", mudlibapi.EMOTE, source)
  source.write(mess)

  room = getattr(source, "_location", None)
  if room != None:
    mess._message = room_text + "\n"
    room.spamroom(mess, [source])

def emote_target(source, target, source_text, target_text, room_text):
  """
  Handles emotes which are executed with a target.

  @param source: the source
  @type  source: Mobile

  @param target: the target
  @type  target: Mobile

  @param source_text: the text the source will see.  ex: "You smile."
  @type  source_text: string

  @param target_text: the text the target will see.  ex: "%s smiles at you."
  @type  target_text: string

  @param room_text: the text the rest of the room sees.  ex: "%s smiles at %t."
  @type  room_text: string
  """
  # write to the source
  mess = message.Message(source_text + "\n", mudlibapi.EMOTE, source)
  source.write(mess)

  # write to the target
  mess._message = target_text + "\n"
  target.write(mess)

  # if the source and target have the same location and the location
  # isn't "None" then we show the room what happened
  source_room = getattr(source, "_location", None)
  target_room = getattr(target, "_location", None)

  if source_room != None and source_room == target_room:
    mess._message = room_text + "\n"
    source_room.spamroom(mess, [source, target])

command_dict = {}
emote_dict = {}

def add_command(name, func):
  """
  Adds a command to the dict.
  """
  global command_dict 
  command_dict[name] = func

def add_emote(name, func):
  """
  Adds an emote to the dict.
  """
  global emote_dict
  emote_dict[name] = func

def load_stuff(stufftype):
  index = __file__.rfind(os.sep)
  if index == -1:
    path = "." + os.sep
  else:
    path = __file__[:index]

  path = path + os.sep + stufftype + "s"
  
  _module_list = glob.glob( os.path.join(path, "*.py"))
  _module_list.sort()

  for mem in _module_list:
    # we skip over all files that start with a _
    # this allows hackers to be working on a module and not have
    # it die every time.
    mem2 = mem[mem.rfind(os.sep)+1:mem.rfind(".")]
    if mem2[0] == "_":
      continue

    try:
      execfile(mem, globals())
    except:
      exported.write_traceback("%s '%s' refuses to load." % (stufftype, mem2))


def cmd_help(pl, myworld, input):
  output = []
  output.append("This is quick help until I write a real help command.")
  output.append("Commands available:")

  listing = myworld._commandmanager.getCommands()
  listing.sort()
  output.append(mudlibapi.columnize(listing, indent=3))
  pl.writeln("\n".join(output) + "\n")
  return 1

command_dict["help"] = cmd_help

def cmd_commandlist(pl, myworld, input):
  listing = myworld._commandmanager.getCommands()
  listing.sort()
  listing = mudlibapi.columnize(listing, indent=3)
  pl.writeln("Global commands available:\n" + listing)
  return 1

command_dict["commandlist"] = cmd_commandlist

def cmd_emotelist(pl, myworld, input):
  listing = myworld._emotemanager.getCommands()
  listing.sort()
  listing = mudlibapi.columnize( listing, indent=3 )
  pl.writeln("Emotes available:\n" + listing)
  return 1

command_dict["emotelist"] = cmd_emotelist


def load(myengine, myworld):
  global command_dict, emote_dict
  load_stuff("emote")
  load_stuff("command")

  for (k, v) in command_dict.items():
    myworld._commandmanager.addCommand(k, v)

  for (k, v) in emote_dict.items():
    myworld._emotemanager.addCommand(k, v)
