# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Holds the Room superclass and subclasses of Room (special rooms).
"""
import object

class Room(object.MudObject):
  """
  This is the room class.  It adds the idea of "exits" to the MudObject
  and it also has a different lifecycle than the rest of the MudObjects.
  Once a room is instantiated, it hangs out in game forever.
  """
  def __init__(self):
    object.MudObject.__init__(self)
    self._names = ["room"]
    self._short = "A blank room."
    self._desc = "A blank room."

    self._contents = []
    self._exits = {}
    self._lookables = {}

  def getDesc(self, mob, name):
    """
    Returns the description for that name for a given mob.

    @param mob: the mobile asking for a description
    @type  mob: mud object

    @param name: the name of the thing to be described
    @type  name: string

    @returns: a description of the thing
    @rtype: string
    """
    name = name.lower()
    if name in self._names:
      return self._desc

    if self._lookables.has_key(name):
      return self._lookables[name]

  def getExits(self):
    """
    Returns the exits that exist in this room.

    @returns: list of the exits available
    @rtype: list of strings
    """
    return self._exits.keys()

  def getExit(self, exitname):
    """
    Returns the exit (if we have one) or None if we don't.
    
    @param exitname: the name of the exit
    @type  exitname: string

    @returns: the room requested or None
    @rtype: Room
    """
    if self._exits.has_key(exitname):
      return self._exits[exitname]

    return None

  def spamroom(self, mess, mob=[]):
    """
    Sends a message to all the members in the room.  This is
    players, npcs, and any objects.

    @param mess: the Message object to write to everyone
    @type  mess: Message

    @param mob: if this is set, we'll skip these mobiles when spamming 
        the room.  i.e. they won't receive this message.
    @type  mob: list of Mobiles
    """
    for mem in self._contents:
      if mem not in mob:
        mem.write(mess)

  def look(self, pl):
    """
    Returns a string that consists of the room description, room
    exits, and contents of the room.

    @param pl: the player looking
    @type  pl: Player

    @returns: the textual description of what was looked at or an 
        error message
    @rtype: string
    """
    output = [self.getShort(pl, self._names[0]), self.getDesc(pl, self._names[0])]

    if self._exits:
      output.append("\nObvious exits: " + ", ".join(self._exits.keys()))
        
    for mem in self._contents:
      if mem != pl:
        output.append(mem.getShort(pl, mem._names[0]))

    return "\n".join(output)
