# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
We hold all time-oriented things in this class: the Heartbeat and
Callout events, the TimedEventManager, and a bunch of other things
as well.
"""
from driver import event, manager, driverapi
import mudlibapi

class HeartbeatEvent(event.Event):
  """
  The heartbeat event handles object heartbeats on the mud.
  It will call the object's "heartbeat" method.
  """
  def __init__(self, source):
    self._source = source

  def __str__(self):
    return "thump"

  def execute(self, my_engine):
    self._source.heartbeat(mudlibapi.get_world())


class CalloutEvent(event.Event):
  """
  CalloutEvents are timed to execute at a given point--but they're
  disassociated with the object that kicked them off.
  """
  def __init__(self, func, *nargs, **args):
    self._func = func
    self._nargs = nargs
    self._args = args

  def execute(self, my_engine):
    self._func(*self._nargs, **self._args)


class TimedEventManager(manager.Manager):
  """
  Handles scheduling events that involve timing--CalloutEvents, 
  HeartbeatEvents, ...
  """
  def __init__(self):
    # scheduled events
    self._events = {}

    # objects registered for heartbeats
    self._hbs = []

    # tells our little thread when to shutdown
    self._shutdownflag = 0

    # which beat we're on
    self._beat = 0

  def startup(self, args):
    """
    We spin off our run thread.
    """
    driverapi.start_thread("timer", self.run)

  def run(self):
    """
    In this thread, we sleep and wake up every second to execute 
    callouts and heartbeats.
    """
    import time
    self._beat = 0
    while not self._shutdownflag:
      try:
        time.sleep(1)

        # handle heartbeats
        if self._beat % 2 == 0:
          for mem in self._hbs:
            HeartbeatEvent(mem).enqueue()

        # handle callouts
        if self._events.has_key(self._beat):
          for mem in self._events[self._beat]:
            mem.enqueue()
          del self._events[self._beat]

        self._beat += 1

        # we recycle the beat every hour
        if self._beat % 3600 == 0:
          self._beat = 0
      except KeyboardInterrupt:
        return
      except SystemExit:
        return
      except:
        driverapi.print_traceback("TimedEventManager: hiccup")


  def callout(self, tickoffset, f, *nargs, **kargs):
    """
    Registers a callout (one-time event) with the scheduler.

    @param tickoffset: how many seconds from now to execute the callout
    @type  tickoffset: int

    @param f: the function to call
    @type  f: function

    @raises: ValueError if the function is not callable
    """
    tick = (self._beat + tickoffset) % 3600
    if callable(f):
      if self._events.has_key(tick):
        self._events[tick].append(CalloutEvent(f, *nargs, **kargs))
      else:
        self._events[tick] = [CalloutEvent(f, *nargs, **kargs)]
    else:
      raise ValueError, "Function is not callable."

        
  def registerHeartbeat(self, ob):
    """
    Registers an object with the list of heart beat objects.  It
    first checks to see if the object is already registered and if
    so doesn't register it again.  The signature for the heart
    beat method should be::

      def heartbeat(self, world)

    @param ob: the object to register
    @type  ob: MudObject
    """
    if ob not in self._hbs:
      self._hbs.append(ob)

  def unregisterHeartbeat(self, ob):
    """
    Unregisters an object from the list of objects who have
    heart beats.

    @param ob: the object to unregister
    @type  ob: MudObject
    """
    if ob in self._hbs:
      self._hbs.remove(ob)

  def executeHeartbeats(self):
    """
    Throw all the registered heartbeat objects as events into
    the queue to be handled.
    """
    if self._hbs:
      for mem in self._hbs:
        HeartbeatEvent(mem).enqueue()

def load(my_engine, my_world):
  tem = TimedEventManager()
  my_engine.addManager("timedevent", tem)
  my_engine.registerHook("startup", tem.startup)
