# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
This has a series of utility functions that aren't related to specific
classes in the application.  They fall into the following classes:
string manipulation, grammar helpers, random generators, and
also text filtering.
"""
import string, re, time

_ANSI_COLOR_REGEXP = re.compile(chr(27) + '\[[0-9;]*[m]')

def color(data, fc=37, bc=40, bold=0):
  """
  Colors a string.
  """
  if bold == 1:
    return "%c[1;%s;%sm%s%c[0m" % (chr(27), str(fc), str(bc), data, chr(27))
  else:
    return "%c[%s;%sm%s%c[0m" % (chr(27), str(fc), str(bc), data, chr(27))



import random
def random_choice(seq):
  """
  Returns a randomly selected element in the sequence.

  @param seq: the sequence of items to randomly choose from
  @type  seq: list

  @returns: an item from the list
  @rtype: any
  """
  if seq:
    return random.choice(seq)
  else:
    return None

def random_number(low, high):
  """
  Generates a random number between low and high such that
  low <= N <= high.  It uses randint right now, but we may
  switch that later on.

  @param low: the low number
  @type  low: int

  @param high: the high number
  @type  high: int

  @returns: the random integer
  @rtype: int
  """
  return random.randint(low, high)

def filter_ansi(text):
  """
  Filters out ansi codes from a given string.

  @param text: a given text string to filter.
  @type  text: string

  @returns: the text without the ansi codes
  @rtype: string
  """
  return re.sub(chr(27) + '\[[0-9;]*[mJ]', '', text)


def filter_cm(text):
  """
  Filters out ^M.  Useful for logging.

  @param text: a given text string to filter.
  @type  text: string

  @returns: the text without the ^M stuff.
  @rtype: string
  """
  return re.sub('\015|\r', '', text)


def chomp(text):
  """
  Removes all '\\r' and '\\n' from the input string.

  @param text: the text to chomp
  @type  text: string

  @returns: the chomped text
  @rtype: string
  """
  text = text.replace("\n", "")
  text = text.replace("\r", "")
  return text


def is_color_token(token):
  """
  Returns whether or not this is a color token.

  @param token: the token to test
  @type  token: string

  @return: 1 if it's a color token, 0 if not
  @rtype: boolean
  """
  if len(token) == 0:
    return 0

  return token[0:2] == chr(27) + "[" and token[-1] == "m"


def fix_color(color):
  """
  Helper function for debugging--it'll fix a color token
  so it's readable in ascii.

  @param color: the color token
  @type  color: string

  @return: the fixed color token
  @rtype: string
  """
  color = color.replace(chr(27), "ESC")
  return color


def split_ansi_from_text(text):
  """
  Takes in a string and returns a list of text and ansi tokens.

  @param text: the text to split into ansi and text tokens
  @type  text: string

  @returns: list of text and ansi tokens
  @rtype: list of strings
  """
  global _ANSI_COLOR_REGEXP

  matchob = _ANSI_COLOR_REGEXP.search(text)
  if matchob:
    textlist = []
    marker = 0
    while matchob:
      (b, e) = matchob.span()

      if marker == b:
        textlist.append(text[b:e])
      else:
        textlist.append(text[marker:b])
        textlist.append(text[b:e])

      marker = e
      matchob = _ANSI_COLOR_REGEXP.search(text, marker)

    # we do this to handle ansi color sequences which are broken
    # between two network chunks
    b = text.rfind(chr(27))

    if b < marker:
      textlist.append(text[marker:])
    else:
      textlist.append(text[marker:b])
      textlist.append(text[b:])

    return textlist

  return [text]


def _find_next_break(token, marker, wrapcount, wraplength):
  """
  Figures out where the next break should be while word-wrapping.

  @param token: the token we're working on
  @type  token: string

  @param marker: the point at which to start looking--the break is
      after this marker in the token
  @type  marker: int

  @param wrapcount: how much is in the line so far
  @type  wrapcount: int

  @param wraplength: the line length to wrap at or under
  @type  wraplength: int

  @returns: index of where to wrap at or -1 if we don't need
      to wrap on this token
  @rtype: int
  """
  # first we check to see to see if we need to find a break
  if len(token) <= marker - wrapcount + wraplength:
    return -1

  # first we look at carriage returns--they're fun and yummy!
  x = token.rfind("\n", marker, marker - wrapcount + wraplength)
  if x != -1:
    return x

  # ok--no carriage returns.  so we try going out wraplength and working
  # to the left for spaces.
  x = token.rfind(" ", marker, marker - wrapcount + wraplength)
  if x != -1:
    return x

  return marker - wrapcount + wraplength


def wrap_text(textlist, wraplength=50, indent=0, firstline=0):
  """
  It takes a block of text and wraps it nicely.  It accounts for
  indenting lines of text, wraplengths, and wrapping around ANSI 
  colors.

  We break on carriage returns (those are easy) and if no carriage
  returns are available we break on spaces.

  If the actual line is longer than the wraplength, then we'll break
  in the line at the wraplength--this will cut words in two.

  @param textlist: either a string of text needing to be formatted and
      wrapped, or a textlist needing to be formatted and wrapped.
  @type  textlist: string or list of strings

  @param wraplength: the maximum length any line can be.  we'll wrap
      at an index equal to or less than this length.
  @type  wraplength: int

  @param indent: how many spaces to indent every line.
  @type  indent: int

  @param firstline: 0 if we shouldn't indent the first line, 1 if we 
      should
  @type  firstline: boolean

  @returns: the wrapped text string
  @rtype: string
  """
  wrapcount = 0           # how much we've got on the line so far
  linecount = 0           # which line we're on

  if wraplength > 2:
    wraplength = wraplength - 2

  # split the formatting from the text
  if type(textlist) == type(''):
    textlist = split_ansi_from_text(textlist)

  for i in range(0, len(textlist)):
    # COLOR TOKEN
    if is_color_token(textlist[i]):
      continue

    # TEXT TOKEN
    marker = 0

    # this handles the offset for not indenting the first line (if that's
    # the sort of thing we're into).
    if firstline:
      x = _find_next_break(textlist[i], marker, wrapcount, wraplength - indent)
    else:
      x = _find_next_break(textlist[i], marker, wrapcount, wraplength)

    # go through finding breaks and sticking in carriage returns and indents
    # and things for this text token
    while x != -1:
      # insert the carriage return, any indent, and lstrip the line as well
      if textlist[i][x] == "\n":
        textlist[i] = (textlist[i][:x+1] + (indent * ' ') + textlist[i][x:].lstrip())
      else:
        textlist[i] = (textlist[i][:x+1] + '\n' + (indent * ' ') + textlist[i][x+1:].lstrip())

      marker = x + indent + 2
      wrapcount = 0

      x = _find_next_break(textlist[i], marker, wrapcount, wraplength - indent)

    wrapcount = len(textlist[i]) - marker + wrapcount


  # this next line joins the list with no separator (GASP!)
  if firstline:
    return (indent * " ") + ''.join(textlist)
  else:
    return ''.join(textlist)


def columnize(textlist, screenwidth=72, indent=0):
  """
  Takes a list of data and converts it into a series of columns
  and rows that are evenly spaced and pretty and stuff.

  @param textlist: the list of tokens to columnize
  @type  textlist: list of strings

  @param screenwidth: the width to wrap against
  @type  screenwidth: int

  @param indent: the amount to indent each line
  @type  indent: int

  @returns: the final formatted string
  @rtype: string
  """
  if screenwidth > 2 + indent:
    screenwidth = screenwidth - 2 - indent

  SPACING = 4
  maxwidth = 0

  for mem in textlist:
    maxwidth = max(maxwidth, len(mem))

  numcols = max(1, (screenwidth + SPACING) / (maxwidth + SPACING))
  numrows = (len(textlist) + numcols - 1) / numcols

  rows = []
  ## We can't just do "rows = ([],) * rows" -- need distinct lists
  for i in range(numrows): 
    rows.append([])

  idx = 0
  for mem in textlist:
    mem = (mem + (' ' * (maxwidth + (SPACING - 1) - len(mem))))

    rows[idx].append(mem)
    idx = (idx + 1) % numrows

  rows = map(string.rstrip, map(string.join, rows))
  return (indent * " ") + string.join(rows, "\n" + (indent * " "))


def find_in_contents(ob, name, index=1):
  """
  Takes a name and tries to find it in the ob's contents.

  @param ob: the object to look in
  @type  ob: MudObject

  @param name: the name of the thing to look for
  @type  name: string

  @param index: we'll return the nth thing we find of that name, or 
      the last one.  this is a cardinal number--not an index into an
      array.  so "1" is the first item, "2" is the second item....
  @type  index: int

  @returns: an object in the ob's contents that matches the name and
      index
  @rtype: MudObject or None
  """
  lastmem = None
  for mem in ob._contents:
    if name in mem._names:
      if index == 1:
        return mem
      else:
        lastmem = mem
        index -= 1

  return lastmem



HISHERITS = { "m": "his", "f": "her", "n": "its"}

def hisherits(mob):
  """
  Returns his/her/its based on the gender of the mobile.  This is a 
  possessive pronoun.  Defaults to neuter usage if the mobile doesn't
  have a _sex attribute.

  @param mob: the mobile in question (though it can handle all objects)
  @type  mob: Mobile

  @returns: his/her/its
  @rtype: string
  """
  return HISHERITS[getattr(mob, "_sex", "n")]


HIMHERITSELF = { "m": "himself", "f": "herself", "n": "itself"}

def himheritself(mob):
  """
  Returns himself/herself/itself based on the gender of the mobile.  
  Defaults to neuter usage if the mobile doesn't have a _sex attribute.

  @param mob: the mobile in question (though it can handle all objects)
  @type  mob: Mobile

  @returns: himself/herself/itself
  @rtype: string
  """
  return HIMHERITSELF[getattr(mob, "_sex", "n")]
