# Stringbean - a simple Python-based mud
# Copyright (C) 2004 Will Guaraldi
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
This is the world module.  Theoretically there should only be one
World instance.  This instance handles much of the data aspects of
the mud and will likely be used to keep track of things like mobiles
in the game, rooms in the game, registered heartbeat objects as well
as dealing with mundane things like loading new rooms into the game
and such.
"""
from driver import manager, event, driverapi
import commandmanager, room, mobile, mudlibapi, time
import traceback

_NAME = 0
_PASSWORD = 1
_SEX = 2
_WORLD = 3

_WELCOME = "Welcome to Neil's Pub!  Type \"help\" if you're lost.\n"
_NAMEPROMPT = "Enter name: "
_PASSWORDPROMPT = "Enter password: "
_SEXPROMPT = "Sex (m/f): "

my_world = None

class World(room.Room, manager.Manager):
  """
  The world is a very special kind of room.  It's the biggest room 
  we have.  It holds all the little rooms inside of it as well
  as manages other various things about the world.  It's also a 
  manager in the engine and handles world things like connects and
  disconnects and such.
  """
  def __init__(self, my_engine):
    # the engine instance
    self._engine = my_engine
    self._options = my_engine._options

    # all the rooms currently in the game
    self._rooms = {}

    # mobiles in the system
    self._mobiles = []

    # mapping of connection -> mobile
    self._conns = {}

    # log for who has logged in today
    self._log = []

    my_engine.addManager("world", self)
    my_engine.registerHook("connect", self.connect_hook)
    my_engine.registerHook("disconnect", self.disconnect_hook)
    my_engine.registerHook("userinput", self.userinput_hook)

    self._commandmanager = commandmanager.CommandManager()
    self._emotemanager = commandmanager.CommandManager()

  def getName(self):
    return "world"

  def getStatus(self):
    return "everything is groovy!"

  def registerMobile(self, mob):
    """
    Registers a mobile with the system.  These are typically players
    and occasionally advanced NPCs.

    @param mob: a Mobile
    @type  mob: Mobile
    """
    self._mobiles.append(mob)

  def unregisterMobile(self, mob):
    """
    Unregisters a mobile with the system.  These are typically players
    and occasionally advanced NPCs.

    @param mob: a Mobile
    @type  mob: Mobile
    """
    if mob in self._mobiles:
      self._mobiles.remove(mob)
 
  def loadRoom(self, roomname):
    """
    Loads a room.  If the room doesn't already exist in memory,
    then we'll load the module it's in, instantiate the room,
    initialize it, and then add it to the list of rooms we
    know about.

    @param roomname: the full class name.  eg. data.tavern.PubEntrance
    @type  roomname: string

    @return: the room instance or None if the room can't be instantiated
    @rtype: Room
    """
    if self._rooms.has_key(roomname):
      return self._rooms[roomname]

    try:
      newroom = driverapi.instantiate(roomname)
      self._rooms[roomname] = newroom
      return newroom

    except Exception, e:
      traceback.print_exc()
      return None

  # --------------------------------------------------
  # communication
  # --------------------------------------------------
  def spamworld(self, mess, mobs=[]):
    """
    Spams the world with a message except mobiles listed in the mobs list.

    @param mess: the message to spam the world with
    @type  mess: Message

    @param mobs: list of the mobiles that shouldn't get spammed
    @type  mobs: list of Mobiles
    """
    for mem in self._mobiles:
      if mem not in mobs:
        mem.write(mess)

  def connect_hook(self, args):
    conn = args[0]

    self._conns[conn] = "login"
    conn.write("Welcome to Neil's tavern!\n")
    conn.write(_WELCOME)
    conn.write(_NAMEPROMPT)

  def disconnect_hook(self, args):
    """
    Disconnects a user from the system.  We don't discriminate
    between linkdead and quit.  If you quit, you quit--se la vie!
    """
    conn = args[0]
    reason = args[1]

    if self._conns.get(conn, "login") != "login":
      pl = self._conns[conn]

      thetime = time.strftime(" - left %I:%M:%S %p", time.localtime())
      self._log.append("  " + pl.getName() + thetime)
      pl.save()

      # if we have a heartbeat, we unregister
      mudlibapi.unregister_hb(pl)
      self.unregisterMobile(pl)

      if pl._location:
        pl._location.spamroom(mudlibapi.Message("%s has left the game.\n" % pl.getName(), mudlibapi.EVENT))

        if pl in pl._location._contents:
          pl._location._contents.remove(pl)

      print("Goodbye %s" % str(pl))
    else:
      print("Goodbye [unknown]")

    conn.write("Goodbye!\n")

  def userinput_hook(self, args):
    """
    Handles incoming user input.
    """
    conn = args[0]
    input = args[1]

    if self._conns[conn] == "login":
      self.login_handler(conn, input)
    else:
      self._conns[conn].handleInput(self, input)

  def login(self, conn):
    """
    Takes a Connection object and creates a Player object for the
    Connection.  Then we move the player to the start room.

    @param conn: the Connection object the player is connected with
    @type  conn: Connection
    """
    pl = mobile.Player(conn, self)
    self._conns[conn] = pl
    room = self.loadRoom(self._options["startroom"])
    if room:
      pl.write("You throw open the door to the Pub and walk on inside....\n")
      pl.setLocation(room)
      pl.write("> ")
      mobs = pl._location._contents

      mess = mudlibapi.Message("%s throws open the door from outside and walks on in!\n" % pl.getName(), mudlibapi.EVENT)
      room.spamroom(mess, [pl])

      mess._message = "You hear a loud thump as someone walks in the door from outside.\n"
      self.spamworld(mess, mobs)

      self.registerMobile(pl)
      mudlibapi.register_hb(pl)

    else:
      pl.write("Error in creating start room.")
      event.HookEvent("disconnect", (conn, driverapi.DISCONNECT_QUIT)).enqueue()
 

  def login_handler(self, conn, input):
    """
    Handles new connections so we can extract a name from them and 
    such.

    @param conn: the Connection object
    @type  conn: Connection

    @param input: what they just typed at us
    @type  input: string
    """
    if conn._state == _NAME:
      # fixme - do a check for other people with that name
      if not input.isalpha():
        conn.write("Ewww--what kind of name is that?  Try again.\n")
        conn.write(_NAMEPROMPT)
        return

      if len(input) > 10:
        conn.write("Dude--choose a shorter name.\n")
        conn.write(_NAMEPROMPT)
        return

      if len(input) < 3:
        conn.write("Dude--choose a longer name.\n")
        conn.write(_NAMEPROMPT)
        return

      conn.write("Your name will be: '%s'.\n" % input)
      conn.write(_SEXPROMPT)
      conn._name = input
      conn._state = _SEX
      return

    if conn._state == _PASSWORD:
      if len(input) < 6 or len(input) > 12:
        conn.write("Dude--choose a password between 6 and 12 characters.\n")
        conn.write(_PASSWORDPROMPT)
        return

      conn.write("Setting password.\n")
      conn.write(_SEXPROMPT)
      conn._password = input
      conn._state = _SEX
      return

    if conn._state == _SEX:
      if input == "m":
        conn.write("Yay--a man!  Sending you off into the world man.\n\n...\n\n")
        conn._state = _WORLD
      elif input == "f":
        conn.write("How cute--a woman!  Now tossing you in the world.\n\n...\n\n")
        conn._state = _WORLD
      else:
        conn.write("That's not a valid choice.\n")
        conn.write(_SEXPROMPT)
        return

      # fixme - need to move them to the start room
      conn._sex = input
      self.login(conn)
